Class: HTTP_Spew::HitNRun
Overview
An even more horrible way to make HTTP requests, just make them without reading a response!
Constant Summary collapse
- RESPONSE =
frozen response
[ 666, { "Content-Length" => "0".freeze, "Content-Type" => "hit/run".freeze }.freeze, [].freeze ].freeze
Instance Attribute Summary
Attributes inherited from Request
Instance Method Summary collapse
-
#read_response ⇒ Object
we close the connection once we’re ready to read the response to save bandwidth.
Methods inherited from Request
#close, #each, #initialize, #resume, #run, #start_sock, #too_big!
Methods included from Headers
Constructor Details
This class inherits a constructor from HTTP_Spew::Request
Instance Method Details
#read_response ⇒ Object
we close the connection once we’re ready to read the response to save bandwidth
17 18 19 20 |
# File 'lib/http_spew/hit_n_run.rb', line 17 def read_response close RESPONSE end |